/*
 * Copyright (C) 2014 Atmel
 *		      Bo Shen <voice.shen@atmel.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/at91_common.h>
#include <asm/arch/at91_rstc.h>
#include <asm/arch/atmel_mpddrc.h>
#include <asm/arch/atmel_usba_udc.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clk.h>
#include <asm/arch/sama5d3_smc.h>
#include <asm/arch/sama5d4.h>
#include <atmel_hlcdc.h>
#include <atmel_mci.h>
#include <i2c.h>
#include <lcd.h>
#include <mmc.h>
#include <net.h>
#include <netdev.h>
#include <nand.h>
#include <spi.h>
#include <version.h>

DECLARE_GLOBAL_DATA_PTR;

#ifdef CONFIG_ATMEL_SPI
int spi_cs_is_valid(unsigned int bus, unsigned int cs)
{
	switch(bus)
	{
	case	0:
		switch(cs)
		{
		case	0:
			return 1;
			break;
		default:
			break;
		}
		break;
	case	1:
		switch(cs)
		{
		case	0:
		case	1:
		case	2:
			return 1;
			break;
		default:
			break;
		}
		break;
	default:
		break;
	}
	return	0;
}

void spi_cs_activate(struct spi_slave *slave)
{
	switch (slave->bus)
	{
	case	0:
		at91_set_pio_output(AT91_PIO_PORTC, 3, 0);
		break;
	case	1:
		switch (slave->cs)
		{
		case	0:
			at91_set_pio_output(AT91_PIO_PORTB, 21, 0);	/* SPI1_CSO NFC */
			break;
		case	1:
			at91_set_pio_output(AT91_PIO_PORTB, 22, 0);	/* SPI1_CS1 BT */
			break;
		case	2:
			at91_set_pio_output(AT91_PIO_PORTB, 23, 0);	/* SPI1_CS2 LCD */
			break;
		default:
			break;
		}
		break;
	default:
		break;
	}

//	uint mode;
//	u8 mode_rx;
//	unsigned int wordlen;
//	unsigned int max_write_size;
//	void *memory_map;
//	u8 option;

//	u8 flags;
//#define SPI_XFER_BEGIN		BIT(0)	/* Assert CS before transfer */
//#define SPI_XFER_END		BIT(1)	/* Deassert CS after transfer */
//#define SPI_XFER_ONCE		(SPI_XFER_BEGIN | SPI_XFER_END)
//#define SPI_XFER_MMAP		BIT(2)	/* Memory Mapped start */
//#define SPI_XFER_MMAP_END	BIT(3)	/* Memory Mapped End */
//#define SPI_XFER_U_PAGE		BIT(4)
}

void spi_cs_deactivate(struct spi_slave *slave)
{
	switch (slave->bus)
	{
	case	0:
		at91_set_pio_output(AT91_PIO_PORTC, 3, 1);
		break;
	case	1:
		switch (slave->cs)
		{
		case	0:
			at91_set_pio_output(AT91_PIO_PORTB, 21, 1);	/* SPI1_CSO NFC */
			break;
		case	1:
			at91_set_pio_output(AT91_PIO_PORTB, 22, 1);	/* SPI1_CS1 BT */
			break;
		case	2:
			at91_set_pio_output(AT91_PIO_PORTB, 23, 1);	/* SPI1_CS2 LCD */
			break;
		default:
			break;
		}
		break;
	default:
		break;
	}	
}

static void sama5d4_xplained_spi0_hw_init(void)
{
	at91_set_a_periph(AT91_PIO_PORTC, 0, 0);	/* SPI0_MISO */
	at91_set_a_periph(AT91_PIO_PORTC, 1, 0);	/* SPI0_MOSI */
	at91_set_a_periph(AT91_PIO_PORTC, 2, 0);	/* SPI0_SPCK */

	at91_set_pio_output(AT91_PIO_PORTC, 3, 1);	/* SPI0_CS0 */

	/* Enable clock */
	at91_periph_clk_enable(ATMEL_ID_SPI0);
}
#endif /* CONFIG_ATMEL_SPI */

#ifdef CONFIG_NAND_ATMEL
static void sama5d4_xplained_nand_hw_init(void)
{
	struct at91_smc *smc = (struct at91_smc *)ATMEL_BASE_SMC;

	at91_periph_clk_enable(ATMEL_ID_SMC);

	/* Configure SMC CS3 for NAND */
	writel(AT91_SMC_SETUP_NWE(1) | AT91_SMC_SETUP_NCS_WR(1) |
	       AT91_SMC_SETUP_NRD(1) | AT91_SMC_SETUP_NCS_RD(1),
	       &smc->cs[3].setup);
	writel(AT91_SMC_PULSE_NWE(2) | AT91_SMC_PULSE_NCS_WR(3) |
	       AT91_SMC_PULSE_NRD(2) | AT91_SMC_PULSE_NCS_RD(3),
	       &smc->cs[3].pulse);
	writel(AT91_SMC_CYCLE_NWE(5) | AT91_SMC_CYCLE_NRD(5),
	       &smc->cs[3].cycle);
	writel(AT91_SMC_TIMINGS_TCLR(2) | AT91_SMC_TIMINGS_TADL(7) |
	       AT91_SMC_TIMINGS_TAR(2)  | AT91_SMC_TIMINGS_TRR(3)   |
	       AT91_SMC_TIMINGS_TWB(7)  | AT91_SMC_TIMINGS_RBNSEL(3)|
	       AT91_SMC_TIMINGS_NFSEL(1), &smc->cs[3].timings);
	writel(AT91_SMC_MODE_RM_NRD | AT91_SMC_MODE_WM_NWE |
	       AT91_SMC_MODE_EXNW_DISABLE |
	       AT91_SMC_MODE_DBW_8 |
	       AT91_SMC_MODE_TDF_CYCLE(3),
	       &smc->cs[3].mode);

	at91_set_a_periph(AT91_PIO_PORTC, 5, 0);	/* D0 */
	at91_set_a_periph(AT91_PIO_PORTC, 6, 0);	/* D1 */
	at91_set_a_periph(AT91_PIO_PORTC, 7, 0);	/* D2 */
	at91_set_a_periph(AT91_PIO_PORTC, 8, 0);	/* D3 */
	at91_set_a_periph(AT91_PIO_PORTC, 9, 0);	/* D4 */
	at91_set_a_periph(AT91_PIO_PORTC, 10, 0);	/* D5 */
	at91_set_a_periph(AT91_PIO_PORTC, 11, 0);	/* D6 */
	at91_set_a_periph(AT91_PIO_PORTC, 12, 0);	/* D7 */
	at91_set_a_periph(AT91_PIO_PORTC, 13, 0);	/* RE */
	at91_set_a_periph(AT91_PIO_PORTC, 14, 0);	/* WE */
	at91_set_a_periph(AT91_PIO_PORTC, 15, 1);	/* NCS */
	at91_set_a_periph(AT91_PIO_PORTC, 16, 1);	/* RDY */
	at91_set_a_periph(AT91_PIO_PORTC, 17, 1);	/* ALE */
	at91_set_a_periph(AT91_PIO_PORTC, 18, 1);	/* CLE */
}
#endif

#ifdef CONFIG_CMD_USB
static void sama5d4_xplained_usb_hw_init(void)
{
	at91_set_pio_output(AT91_PIO_PORTE, 11, 1);
	at91_set_pio_output(AT91_PIO_PORTE, 14, 1);
}
#endif

#ifdef CONFIG_LCD

typedef	enum
{
	ILI_CONTROL_TYPE_COMMAND,
	ILI_CONTROL_TYPE_DATA
} ILIControlType_t;

#define	ILI9341_LCD_CS_LOW()		at91_set_pio_value(AT91_PIO_PORTB, 23, 0)
#define	ILI9341_LCD_CS_HIGH()		at91_set_pio_value(AT91_PIO_PORTB, 23, 1)

#define	ILI9341_LCD_CLOCK_LOW()		at91_set_pio_value(AT91_PIO_PORTB, 20, 0)
#define	ILI9341_LCD_CLOCK_HIGH()	at91_set_pio_value(AT91_PIO_PORTB, 20, 1)

#define	ILI9341_LCD_DATA_LOW()		at91_set_pio_value(AT91_PIO_PORTB, 19, 0)
#define	ILI9341_LCD_DATA_HIGH()		at91_set_pio_value(AT91_PIO_PORTB, 19, 1)

#define	ILI9341_LCD_RESET_LOW()		at91_set_pio_value(AT91_PIO_PORTA, 1, 0)
#define	ILI9341_LCD_RESET_HIGH()	at91_set_pio_value(AT91_PIO_PORTA, 1, 1)

static	void	ili9341_control_delay(void)
{
	uint32_t	Loop;

	for (Loop = 0; Loop < 1500; Loop ++)
	{
		;
	}
}

static	void	ili9341_control_port(ILIControlType_t Type, uint8_t Value)
{
	uint32_t	Index;

	/* >> Start */
	ILI9341_LCD_CS_HIGH();
	ili9341_control_delay();

	ILI9341_LCD_CLOCK_LOW();
	ili9341_control_delay();

	ILI9341_LCD_CLOCK_HIGH();
	ili9341_control_delay();

	ILI9341_LCD_CLOCK_LOW();
	ILI9341_LCD_CS_LOW();

	/* >> Command or Data */
	if(Type == ILI_CONTROL_TYPE_COMMAND)
	{
		ILI9341_LCD_DATA_LOW();
	}
	else
	{
		ILI9341_LCD_DATA_HIGH();
	}
	ili9341_control_delay();

	ILI9341_LCD_CLOCK_HIGH();
	ili9341_control_delay();

	ILI9341_LCD_CLOCK_LOW();

	/* >> Value */
	for(Index = 0; Index < 8; Index++)
	{
		if((Value & 0x80) != 0)
		{
			ILI9341_LCD_DATA_HIGH();
		}
		else
		{
			ILI9341_LCD_DATA_LOW();
		}
		ili9341_control_delay();
		ILI9341_LCD_CLOCK_HIGH();

		ili9341_control_delay();
		ILI9341_LCD_CLOCK_LOW();
		Value <<= 1;
	}

	/* >> Stop */
	ILI9341_LCD_CS_HIGH();
	ILI9341_LCD_CLOCK_HIGH();
}

void	ili9341_initialize(void)
{
	//************* Start Initial Sequence **********//
	// Power control B
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xCF);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xC1);	//	CB
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x30);

	// Power on sequence control
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xED);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x64);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x03);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x12);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x81);

	// Driver timing control A
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xE8);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x85);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x10);
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x78);	//	7A
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x7A);	//	7A

	// Power control A
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xCB);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x39);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x2C);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x34);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x04);	//	02

	// Pump ratio control
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xF7);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x20);


//	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x0C);
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x66);		// 18Bit


	// COLMOD: Pixel Format Set
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x3A);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x66);		// 18Bit	// --> DPI[2:0]

	// Driver timing control C
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xEA);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);


	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x06);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x66);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x66);

	// RGB I/F
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xB0);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xC0);		// --> RCM[1:0] 0xC0 is DE mode 156Page	//	E0
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xE0);		// --> RCM[1:0] 0xE0 is Sync mode 156Page

	// frame rate control
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xB1);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x1B);		// 1b:70hz  1f:61hz
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x1C);		// 1b:70hz  1f:61hz  1c:68Hz

	// display inversion control
	// ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xB4);
	// ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);

	// Display Function Control
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xB6);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x02);	//	0A
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xA2);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x27);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x04);


	// Power Control 1
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xC0);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x22);		//	22->1B	//	21

	// Power Control 2
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xC1);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x10);		//	13->10	//	11

	// VCOM Control 1
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xC5);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x44);		// VCMH	//	3F
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x42);		// VCML	//	3C

	// VCOM Control 2
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xC7);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xB7);		// VCMF CA->B7	//	AF

	// Memory Access Control
#if 0	// LCD Rotation
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x36);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xA8);
#else
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x36);
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x08);		//	00
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);		//	00
#endif

	// 3Gamma Function Disable
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xF2);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);

	// interface control
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xF6);
//	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x01);	//	00
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);	//	00
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x10);	//	00
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x06);		// --> RIM[0] 194Page	//	07 -> 06

	// Gamma curve selected
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x26);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x01);

	// Set Gamma
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xE0);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0D);	//	0F
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x2A);	//	23
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x27);	//	20
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0A);	//	0C
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0B);	//	0F
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x05);	//	09
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x49);	//	4E
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x79);	//	A8
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x3D);	//	3D
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0D);	//	0B
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x18);	//	15
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0C);	//	06
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x10);	//	0E
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x06);	//	08
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);

	// Set Gamma
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0xE1);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x02);	//	00
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0B);	//	1C
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0D);	//	1F
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x04);	//	03
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x11);	//	10
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x06);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x18);	//	31
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x16);	//	57
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x2B);	//	42
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x03);	//	04
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x06);	//	0A
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x07);	//	09
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x2A);	//	31
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x2E);	//	37
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x0F);

	// Sleep Out
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x11);
	ili9341_control_delay();

	// Display ON
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x29);



//	LCD_SPI_WriteComm(0xb0);
//	LCD_SPI_WriteData(0x80);

//	LCD_SPI_WriteComm(0xB5);
//	LCD_SPI_WriteData(0x02);
//	LCD_SPI_WriteData(0x02);
//	LCD_SPI_WriteData(0x0A);
//	LCD_SPI_WriteData(0x14);

	// Ilde mode OFF
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x38);

#if 0	// LCD Rotation
	// Column Address Set
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x2A);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x01);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x3F);

	// Page Address Set
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x2B);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0x00);
	ili9341_control_port(ILI_CONTROL_TYPE_DATA, 0xEF);

	// Memory Write
	ili9341_control_port(ILI_CONTROL_TYPE_COMMAND, 0x2C);
#endif
}

/*ILI9341*/
vidinfo_t panel_info = {
	.vl_col = 480,
	.vl_row = 272,
	.vl_clk = 9000000,
	.vl_bpix = LCD_BPP,
	.vl_tft = 1,
	.vl_sync = 3,				/* 00 = active High,  11 = Active Low Horizontal / vertical sync */
	/* Horizontal control register. */
	.vl_hsync_len = 10,
	.vl_left_margin = 20,
	.vl_right_margin = 10,
	/* Vertical control register. */
	.vl_vsync_len = 2,
	.vl_upper_margin = 2,
	.vl_lower_margin = 4,
	.mmio = ATMEL_BASE_LCDC,
};

/* No power up/down pin for the LCD pannel */
void lcd_enable(void)	{ /* Empty! */ }
void lcd_disable(void)	{ /* Empty! */ }

unsigned int has_lcdc(void)
{
	return 1;
}

static void sama5d4_xplained_lcd_hw_init(void)
{
	at91_set_a_periph(AT91_PIO_PORTA, 24, 0);	/* LCDPWM */
//	at91_set_a_periph(AT91_PIO_PORTA, 25, 0);	/* LCDDISP */
	at91_set_a_periph(AT91_PIO_PORTA, 26, 0);	/* LCDVSYNC */
	at91_set_a_periph(AT91_PIO_PORTA, 27, 0);	/* LCDHSYNC */
	at91_set_a_periph(AT91_PIO_PORTA, 28, 0);	/* LCDDOTCK */
	at91_set_a_periph(AT91_PIO_PORTA, 29, 0);	/* LCDDEN */

//	at91_set_a_periph(AT91_PIO_PORTA,  0, 0);	/* LCDD0 */

//	at91_set_a_periph(AT91_PIO_PORTA,  1, 0);	/* LCDD1 */
	at91_set_pio_pullup(AT91_PIO_PORTA, 1, 1);	/* LCD RESET */
	at91_set_pio_output(AT91_PIO_PORTA, 1, 0);	/* LCD RESET */

	at91_set_a_periph(AT91_PIO_PORTA,  2, 0);	/* LCDD2 */
	at91_set_a_periph(AT91_PIO_PORTA,  3, 0);	/* LCDD3 */
	at91_set_a_periph(AT91_PIO_PORTA,  4, 0);	/* LCDD4 */
	at91_set_a_periph(AT91_PIO_PORTA,  5, 0);	/* LCDD5 */
	at91_set_a_periph(AT91_PIO_PORTA,  6, 0);	/* LCDD6 */
	at91_set_a_periph(AT91_PIO_PORTA,  7, 0);	/* LCDD7 */

//	at91_set_a_periph(AT91_PIO_PORTA,  8, 0);	/* LCDD9 */
//	at91_set_a_periph(AT91_PIO_PORTA,  9, 0);	/* LCDD8 */
	at91_set_a_periph(AT91_PIO_PORTA, 10, 0);	/* LCDD10 */
	at91_set_a_periph(AT91_PIO_PORTA, 11, 0);	/* LCDD11 */
	at91_set_a_periph(AT91_PIO_PORTA, 12, 0);	/* LCDD12 */
	at91_set_a_periph(AT91_PIO_PORTA, 13, 0);	/* LCDD13 */
	at91_set_a_periph(AT91_PIO_PORTA, 14, 0);	/* LCDD14 */
	at91_set_a_periph(AT91_PIO_PORTA, 15, 0);	/* LCDD15 */

//	at91_set_a_periph(AT91_PIO_PORTA, 16, 0);	/* LCDD16 */
//	at91_set_a_periph(AT91_PIO_PORTA, 17, 0);	/* LCDD17 */
	at91_set_a_periph(AT91_PIO_PORTA, 18, 0);	/* LCDD18 */
	at91_set_a_periph(AT91_PIO_PORTA, 19, 0);	/* LCDD19 */
	at91_set_a_periph(AT91_PIO_PORTA, 20, 0);	/* LCDD20 */
	at91_set_a_periph(AT91_PIO_PORTA, 21, 0);	/* LCDD21 */
	at91_set_a_periph(AT91_PIO_PORTA, 22, 0);	/* LCDD22 */
	at91_set_a_periph(AT91_PIO_PORTA, 23, 0);	/* LCDD23 */

	at91_set_pio_input(AT91_PIO_PORTB, 18, 0/*Not Used Pull-Up*/);	/* LCD MISO */
	at91_set_pio_output(AT91_PIO_PORTB, 19, 1);	/* LCD MOSI */
	at91_set_pio_output(AT91_PIO_PORTB, 20, 1);	/* LCD CLK */
	at91_set_pio_output(AT91_PIO_PORTB, 23, 1);	/* LCD CS */

	ILI9341_LCD_CS_HIGH();
	ILI9341_LCD_CLOCK_HIGH();
	ILI9341_LCD_DATA_HIGH();

	ILI9341_LCD_RESET_HIGH();
	ili9341_control_delay();

	ILI9341_LCD_RESET_LOW();
	ili9341_control_delay();

	ILI9341_LCD_RESET_HIGH();
	ili9341_control_delay();

	ili9341_initialize();

	/* Enable clock */
	at91_periph_clk_enable(ATMEL_ID_LCDC);
}

#ifdef CONFIG_LCD_INFO
void lcd_show_board_info(void)
{
	ulong dram_size, nand_size;
	int i;
	char temp[32];

	lcd_printf("%s\n", U_BOOT_VERSION);
	lcd_printf("2017\n");
	lcd_printf("Value Innovation\n");
	lcd_printf("%s CPU at %s MHz\n", get_cpu_name(),
		   strmhz(temp, get_cpu_clk_rate()));

	dram_size = 0;
	for (i = 0; i < CONFIG_NR_DRAM_BANKS; i++)
		dram_size += gd->bd->bi_dram[i].size;

	nand_size = 0;
#ifdef CONFIG_NAND_ATMEL
	for (i = 0; i < CONFIG_SYS_MAX_NAND_DEVICE; i++)
		nand_size += nand_info[i].size;
#endif
	lcd_printf("%ld MB SDRAM, %ld MB NAND\n",
		   dram_size >> 20, nand_size >> 20);
}
#endif /* CONFIG_LCD_INFO */

#endif /* CONFIG_LCD */

#ifdef CONFIG_GENERIC_ATMEL_MCI
void sama5d4_xplained_mci1_hw_init(void)
{
	at91_set_c_periph(AT91_PIO_PORTE, 19, 1);	/* MCI1 CDA */
	at91_set_c_periph(AT91_PIO_PORTE, 20, 1);	/* MCI1 DA0 */
	at91_set_c_periph(AT91_PIO_PORTE, 21, 1);	/* MCI1 DA1 */
	at91_set_c_periph(AT91_PIO_PORTE, 22, 1);	/* MCI1 DA2 */
	at91_set_c_periph(AT91_PIO_PORTE, 23, 1);	/* MCI1 DA3 */
	at91_set_c_periph(AT91_PIO_PORTE, 18, 0);	/* MCI1 CLK */

	/*
	 * As the mci io internal pull down is too strong, so if the io needs
	 * external pull up, the pull up resistor will be very small, if so
	 * the power consumption will increase, so disable the interanl pull
	 * down to save the power.
	 */
	at91_set_pio_pulldown(AT91_PIO_PORTE, 18, 0);
	at91_set_pio_pulldown(AT91_PIO_PORTE, 19, 0);
	at91_set_pio_pulldown(AT91_PIO_PORTE, 20, 0);
	at91_set_pio_pulldown(AT91_PIO_PORTE, 21, 0);
	at91_set_pio_pulldown(AT91_PIO_PORTE, 22, 0);
	at91_set_pio_pulldown(AT91_PIO_PORTE, 23, 0);

	/* Enable clock */
	at91_periph_clk_enable(ATMEL_ID_MCI1);
}

int board_mmc_init(bd_t *bis)
{
	/* Enable the power supply */
	at91_set_pio_output(AT91_PIO_PORTE, 4, 0);

	return atmel_mci_init((void *)ATMEL_BASE_MCI1);
}
#endif /* CONFIG_GENERIC_ATMEL_MCI */

#ifdef CONFIG_MACB
void sama5d4_xplained_macb0_hw_init(void)
{
	at91_set_a_periph(AT91_PIO_PORTB, 0, 0);	/* ETXCK_EREFCK */
	at91_set_a_periph(AT91_PIO_PORTB, 6, 0);	/* ERXDV */
	at91_set_a_periph(AT91_PIO_PORTB, 8, 0);	/* ERX0 */
	at91_set_a_periph(AT91_PIO_PORTB, 9, 0);	/* ERX1 */
	at91_set_a_periph(AT91_PIO_PORTB, 7, 0);	/* ERXER */
	at91_set_a_periph(AT91_PIO_PORTB, 2, 0);	/* ETXEN */
	at91_set_a_periph(AT91_PIO_PORTB, 12, 0);	/* ETX0 */
	at91_set_a_periph(AT91_PIO_PORTB, 13, 0);	/* ETX1 */
	at91_set_a_periph(AT91_PIO_PORTB, 17, 0);	/* EMDIO */
	at91_set_a_periph(AT91_PIO_PORTB, 16, 0);	/* EMDC */

	/* Enable clock */
	at91_periph_clk_enable(ATMEL_ID_GMAC0);
}
#endif

static void sama5d4_xplained_serial3_hw_init(void)
{
	at91_set_b_periph(AT91_PIO_PORTE, 17, 0);	/* TXD3 */
	at91_set_b_periph(AT91_PIO_PORTE, 16, 0);	/* RXD3 */

	/* Enable clock */
	at91_periph_clk_enable(ATMEL_ID_USART3);
}


//#define	OIL_STATION_PRINTER
#define	OIL_STATION_AP
//#define	OIL_STATION_HANDY

static void sama5d4_xplained_led_hw_init(void)
{
#ifdef	OIL_STATION_PRINTER
	at91_set_pio_pullup(AT91_PIO_PORTA, 25, 1);	/* LED nBusy */
	at91_set_pio_pullup(AT91_PIO_PORTB, 14, 1);	/* LED BUSY */
	at91_set_pio_pullup(AT91_PIO_PORTB, 15, 1);	/* LED NO Paper */

	at91_set_pio_output(AT91_PIO_PORTA, 25, 0);	/* LED nBusy */
	at91_set_pio_output(AT91_PIO_PORTB, 14, 0);	/* LED BUSY */
	at91_set_pio_output(AT91_PIO_PORTB, 15, 0);	/* LED NO Paper */
#else	/* OIL_STATION_PRINTER */
	#ifdef	OIL_STATION_AP
	at91_set_pio_pullup(AT91_PIO_PORTB, 14, 1);	/* LED STATUS */
	at91_set_pio_pullup(AT91_PIO_PORTB, 15, 1);	/* LED ERROR */

	at91_set_pio_output(AT91_PIO_PORTB, 14, 0);	/* LED STATUS */
	at91_set_pio_output(AT91_PIO_PORTB, 15, 0);	/* LED ERROR */
	#else	/* OIL_STATION_AP */
		#ifdef	OIL_STATION_HANDY
	at91_set_pio_pullup(AT91_PIO_PORTA, 25, 1);	/* LED nBusy */
	at91_set_pio_pullup(AT91_PIO_PORTB, 14, 1);	/* LED BUSY */
	at91_set_pio_pullup(AT91_PIO_PORTB, 15, 1);	/* LED NO Paper */

	at91_set_pio_output(AT91_PIO_PORTA, 25, 0);	/* LED nBusy */
	at91_set_pio_output(AT91_PIO_PORTB, 14, 0);	/* LED BUSY */
	at91_set_pio_output(AT91_PIO_PORTB, 15, 0);	/* LED NO Paper */
		#endif	/* OIL_STATION_HANDY */
	#endif	/* OIL_STATION_AP */
#endif	/* OIL_STATION_PRINTER */

}

int board_early_init_f(void)
{
	at91_periph_clk_enable(ATMEL_ID_PIOA);
	at91_periph_clk_enable(ATMEL_ID_PIOB);
	at91_periph_clk_enable(ATMEL_ID_PIOC);
	at91_periph_clk_enable(ATMEL_ID_PIOD);
	at91_periph_clk_enable(ATMEL_ID_PIOE);

	sama5d4_xplained_serial3_hw_init();

	return 0;
}

int board_init(void)
{
	/* adress of boot parameters */
	gd->bd->bi_boot_params = CONFIG_SYS_SDRAM_BASE + 0x100;

#ifdef CONFIG_ATMEL_SPI
	sama5d4_xplained_spi0_hw_init();
#endif
#ifdef CONFIG_NAND_ATMEL
	sama5d4_xplained_nand_hw_init();
#endif
#ifdef CONFIG_GENERIC_ATMEL_MCI
	sama5d4_xplained_mci1_hw_init();
#endif
#ifdef CONFIG_MACB
	sama5d4_xplained_macb0_hw_init();
#endif
#ifdef CONFIG_LCD
	sama5d4_xplained_lcd_hw_init();
#endif
#ifdef CONFIG_CMD_USB
	sama5d4_xplained_usb_hw_init();
#endif
#ifdef CONFIG_USB_GADGET_ATMEL_USBA
	at91_udp_hw_init();
#endif

	sama5d4_xplained_led_hw_init();

	return 0;
}

int dram_init(void)
{
	gd->ram_size = get_ram_size((void *)CONFIG_SYS_SDRAM_BASE,
				    CONFIG_SYS_SDRAM_SIZE);
	return 0;
}

#ifdef CONFIG_CMD_I2C
static int set_ethaddr_from_eeprom(void)
{
	const int ETH_ADDR_LEN = 6;
	unsigned char ethaddr[ETH_ADDR_LEN];
	const char * ETHADDR_NAME = "ethaddr";

	if (getenv(ETHADDR_NAME))
		return 0;

	if (i2c_set_bus_num(0)) {
		printf("i2c bus 0 is not valid\n");
		return -1;
	}

	if (i2c_read(CONFIG_AT24MAC_ADDR,
		     CONFIG_AT24MAC_REG, 1, ethaddr, ETH_ADDR_LEN)) {
		printf("Failed to read ethernet address from EEPROM\n");
		return -1;
	}

	if (!is_valid_ethaddr(ethaddr)) {
		printf("The ethernet address read from EEPROM is not valid!\n");
		return -1;
	}

	return eth_setenv_enetaddr(ETHADDR_NAME, ethaddr);
}
#else
static int set_ethaddr_from_eeprom(void)
{
	return 0;
}
#endif

int board_eth_init(bd_t *bis)
{
	int rc = 0;

#ifdef CONFIG_MACB
	rc = macb_eth_initialize(0, (void *)ATMEL_BASE_GMAC0, 0x00);
	if (rc)
		printf("GMAC register failed\n");
	else
		set_ethaddr_from_eeprom();
#endif

#ifdef CONFIG_USB_GADGET_ATMEL_USBA
	usba_udc_probe(&pdata);
#ifdef CONFIG_USB_ETH_RNDIS
	usb_eth_initialize(bis);
#endif
#endif

	return rc;
}

/* SPL */
#ifdef CONFIG_SPL_BUILD
void spl_board_init(void)
{
#ifdef CONFIG_SYS_USE_MMC
	sama5d4_xplained_mci1_hw_init();
#elif CONFIG_SYS_USE_NANDFLASH
	sama5d4_xplained_nand_hw_init();
#elif CONFIG_SYS_USE_SERIALFLASH
	sama5d4_xplained_spi0_hw_init();
#endif
}

static void ddr2_conf(struct atmel_mpddrc_config *ddr2)
{
	ddr2->md = (ATMEL_MPDDRC_MD_DBW_16_BITS | ATMEL_MPDDRC_MD_DDR2_SDRAM);

	ddr2->cr = (ATMEL_MPDDRC_CR_NC_COL_10 |
		    ATMEL_MPDDRC_CR_NR_ROW_14 |
		    ATMEL_MPDDRC_CR_CAS_DDR_CAS3 |
		    ATMEL_MPDDRC_CR_NB_8BANKS |
		    ATMEL_MPDDRC_CR_NDQS_DISABLED |
		    ATMEL_MPDDRC_CR_DECOD_INTERLEAVED |
		    ATMEL_MPDDRC_CR_UNAL_SUPPORTED);

	ddr2->rtr = 0x2b0;

	ddr2->tpr0 = (8 << ATMEL_MPDDRC_TPR0_TRAS_OFFSET |
		      3 << ATMEL_MPDDRC_TPR0_TRCD_OFFSET |
		      3 << ATMEL_MPDDRC_TPR0_TWR_OFFSET |
		      10 << ATMEL_MPDDRC_TPR0_TRC_OFFSET |
		      3 << ATMEL_MPDDRC_TPR0_TRP_OFFSET |
		      2 << ATMEL_MPDDRC_TPR0_TRRD_OFFSET |
		      2 << ATMEL_MPDDRC_TPR0_TWTR_OFFSET |
		      2 << ATMEL_MPDDRC_TPR0_TMRD_OFFSET);

	ddr2->tpr1 = (2 << ATMEL_MPDDRC_TPR1_TXP_OFFSET |
		      200 << ATMEL_MPDDRC_TPR1_TXSRD_OFFSET |
		      25 << ATMEL_MPDDRC_TPR1_TXSNR_OFFSET |
		      23 << ATMEL_MPDDRC_TPR1_TRFC_OFFSET);

	ddr2->tpr2 = (7 << ATMEL_MPDDRC_TPR2_TFAW_OFFSET |
		      2 << ATMEL_MPDDRC_TPR2_TRTP_OFFSET |
		      3 << ATMEL_MPDDRC_TPR2_TRPA_OFFSET |
		      2 << ATMEL_MPDDRC_TPR2_TXARDS_OFFSET |
		      8 << ATMEL_MPDDRC_TPR2_TXARD_OFFSET);
}

void mem_init(void)
{
	struct atmel_mpddrc_config ddr2;

	ddr2_conf(&ddr2);

	/* Enable MPDDR clock */
	at91_periph_clk_enable(ATMEL_ID_MPDDRC);
	at91_system_clk_enable(AT91_PMC_DDR);

	/* DDRAM2 Controller initialize */
	ddr2_init(ATMEL_BASE_MPDDRC, ATMEL_BASE_DDRCS, &ddr2);
}

void at91_pmc_init(void)
{
	u32 tmp;

	tmp = AT91_PMC_PLLAR_29 |
	      AT91_PMC_PLLXR_PLLCOUNT(0x3f) |
	      AT91_PMC_PLLXR_MUL(87) |
	      AT91_PMC_PLLXR_DIV(1);
	at91_plla_init(tmp);

	at91_pllicpr_init(AT91_PMC_IPLL_PLLA(0x0));

	tmp = AT91_PMC_MCKR_H32MXDIV |
	      AT91_PMC_MCKR_PLLADIV_2 |
	      AT91_PMC_MCKR_MDIV_3 |
	      AT91_PMC_MCKR_CSS_PLLA;
	at91_mck_init(tmp);
}
#endif
